var buttonSize = 80;

function World() {
  this.mouse = {
    x: NaN,
    y: NaN,
  };
  this.worldMouse = {
    x: NaN,
    y: NaN,
  };
  this.camera = new Camera();
  this.camera.center(0.5,0,2);
  
var INTRO_FADE_IN_TIME = 2000;
var INTRO_WAIT_TIME = 3000;
var INTRO_FADE_OUT_TIME = 5000;

  var delay = INTRO_FADE_IN_TIME + INTRO_WAIT_TIME + INTRO_FADE_OUT_TIME;
  this.factories = [];
  this.events = [
    [delay, Factory.A(0,0)],
    [delay, Factory.A(1,0).requires("A",4)]
  ];
  this.effects = [];
  this.effect(new Intro());
  this.background = new Background(this.camera, 0.1,0.15,0.2);
  
  this.gameStep = 0;
  this.frames = [];
  this.frames.nextIndex = 0;
  
}
World.prototype.onEnter = function () {
  MUSIC.stopAll();
  MUSIC.play(Game.audioContext, MUSIC.main.get());
};
World.prototype.step = function (elapsed) {
  if (this.background) this.background.step(elapsed);
  this.camera.step(elapsed);
  this.camera.pointToWorld(this.mouse.x,this.mouse.y,this.worldMouse);
  
  var thisWorld = this;
  var allActive = true;
  
  this.hoverMute = this.mouse.x > Game.WIDTH - buttonSize && this.mouse.y > Game.HEIGHT - buttonSize;
  this.hoverMuteMusic = !this.hoverMute && this.mouse.x > Game.WIDTH - 2 * buttonSize && this.mouse.y > Game.HEIGHT - buttonSize;  
  this.factories.forEach(function (f) { 
    f.step(elapsed, thisWorld.worldMouse); 
    if (f.components) allActive = false;
  });
  this.effects.forEach(function (e) { 
    e.step(elapsed); 
  });
  
  if (allActive && this.events.length === 0) {
    switch(++this.gameStep) {
      case 1:
        this.events.push([1000, Factory.AA(0.5,1).requires("A",8)]);
        this.camera.smooth().center(0.5,0.5,2);
      break;
      case 2:
        this.events.push([1000, Factory.A(1.5,1).requires("AA",4)]);
        this.camera.smooth().center(0.75,0.5,2.5);
      break;
      case 3:
        this.events.push([1000, Factory.A(-0.5,1).requires("AAA",4)]);
        this.events.push([1250, Factory.AAA(0,2).requires("AA",8)]);
        this.events.push([1500, Factory.AA(1,2).requires("A",8)]);
        this.camera.smooth().center(0.5,1,3);
      break;
      case 4:
        this.events.push([1000, Factory.A(-1,2).requires("A",16).requires("AAA",2)]);
        this.events.push([1250, Factory.A(1.5,3).requires("AA",8)]);
        this.events.push([1500, Factory.AAA(0.5,3).requires("AA",8).requires("AAA",2)]);
        this.events.push([1750, Factory.AAAA(-0.5,3).requires("AAA",4)]);
        this.events.push([2000, new Challenge(-1.5,3,"B").requires("AAAA",4)]);
        this.camera.smooth().center(0,1.5,4);
      break;
      case 5:
        this.camera.smooth(2000).center(-1.5,3,6);
        this.events.push([SUCTION_FADE_IN_TIME, function () {
          thisWorld.camera.center(0,0.25,2.25);
          thisWorld.factories = [];
          thisWorld.background = new Background(thisWorld.camera, 0.15,0.2,0.1);
        }]);
        this.events.push([SUCTION_FADE_IN_TIME+SUCTION_BLANK_TIME+4000, Factory.B(0,0)]);
        this.events.push([SUCTION_FADE_IN_TIME+SUCTION_BLANK_TIME+4750, Factory.A(0.8,0.5).requires("B",4)]);
        this.events.push([SUCTION_FADE_IN_TIME+SUCTION_BLANK_TIME+5000, Factory.B(-0.8,0.5).requires("B",4)]);
      break;
      case 6:
        this.camera.smooth().center(0,0.75,2.5);
        this.events.push([1000, Factory.A(0.5,1.5).requires("BB",3)]);
        this.events.push([1250, Factory.B(-0.5,1.5).requires("BB",3)]);
        this.events.push([1500, Factory.BB(0,0.75).requires("B",12).requires("A",4)]);
      break;
      case 7:
        this.camera.smooth().center(0,0.9,3);
        this.events.push([1000, Factory.A(-0.6,-0.25).requires("A",8)]);
        this.events.push([1250, Factory.AA(-1.2,1.2).requires("BB",4)]);
        this.events.push([1500, Factory.AAA(-1.1,2).requires("AAA",4).requires("BB",4)]);
        this.events.push([1750, Factory.A(0.6,-0.25).requires("BB",6)]);
        this.events.push([2000, Factory.AA(1.2,1.2).requires("B",18).requires("A",16)]);
        this.events.push([2250, Factory.AAA(1.1,2).requires("BB",4).requires("AA",4)]);
      break;
      case 8:
        this.camera.smooth().center(0,1.25,4);
        this.events.push([1000, Factory.A(1.3,0).requires("AA",6).requires("A",6)]);
        this.events.push([1250, Factory.A(1.5,2.75).requires("AAA",4).requires("A",4)]);
        this.events.push([1500, Factory.BBB(0,2.5).requires("BB",8).requires("AAA",6)]);
        this.events.push([1750, Factory.A(-1.5,2.75).requires("AAA",4).requires("BB",8)]);
        this.events.push([2000, Factory.A(-1.3,0).requires("AA",6).requires("B",8)]);
        this.events.push([2250, Factory.BBBB(-1.7,2).requires("BBB",2)]);
      break;
      case 9:
        this.camera.smooth().center(-0.7,1.7,5);
        this.events.push([1000, Factory.B(-1.7,0.6).requires("BBB",3)]);
        this.events.push([1200, Factory.B(-2,-0.5).requires("A",6)]);
        this.events.push([1400, Factory.B(-2.6,-0.25).requires("AA",6)]);
        this.events.push([1600, Factory.A(-2.8,0.8).requires("AAA",6)]);
        this.events.push([1800, Factory.A(-2.5,1.4).requires("BB",6)]);
        this.events.push([2000, Factory.BB(-2.25,0.4).requires("BBBB",3)]);
        this.events.push([2200, Factory.BB(-2.25,2.4).requires("BBBB",15).requires("BBB",2)]);
      break;
      case 10:
        this.events.push([1000, Factory.B(-2.8,2).requires("BBB",3)]);
        this.events.push([1200, Factory.B(-2.9,2.6).requires("BBBB",12)]);
        this.events.push([1400, Factory.B(-2.6,3.2).requires("BBB",3)]);
        this.events.push([1600, Factory.A(-2,3.6).requires("BBBB",12)]);
        this.events.push([1800, Factory.A(-1.2,3.8).requires("AAA",4)]);
        this.events.push([2000, Factory.AAAA(0,3.25).requires("BBBB",40).requires("BBB",4)]);
        this.events.push([2200, new Challenge(1.25,3.5,"C").requires("BBBB",3).requires("BBB",3).requires("AAAA",3)]);
      break;
      case 11:
        this.camera.smooth(2000).center(1.25,3.5,6);
        this.events.push([SUCTION_FADE_IN_TIME, function () {
          thisWorld.camera.center(0,0,2.5);
          thisWorld.factories = [];
          thisWorld.background = new Background(thisWorld.camera, 0.2,0.1,0.15);
        }]);
        this.events.push([SUCTION_FADE_IN_TIME+SUCTION_BLANK_TIME+4000, Factory.C(-0.4,-0.8)]);
        this.events.push([SUCTION_FADE_IN_TIME+SUCTION_BLANK_TIME+4200, Factory.C(0.4,-0.7).requires("C",1)]);
        this.events.push([SUCTION_FADE_IN_TIME+SUCTION_BLANK_TIME+4400, Factory.B(0.9,0.1).requires("C",1)]);
        this.events.push([SUCTION_FADE_IN_TIME+SUCTION_BLANK_TIME+4600, Factory.A(0.2,0.8).requires("C",1)]);
        this.events.push([SUCTION_FADE_IN_TIME+SUCTION_BLANK_TIME+4800, Factory.A(-0.4,0.9).requires("C",1)]);
        this.events.push([SUCTION_FADE_IN_TIME+SUCTION_BLANK_TIME+5000, Factory.CC(0,0).requires("A",3).requires("B",6).requires("C",2)]);
      break;
      case 12:
        this.camera.smooth(2000).center(-0.4,0.3,3.2);
        this.events.push([1000, Factory.AA(-0.8,-0.1).requires("CC",6)]);
        this.events.push([1250, Factory.BB(0,1.6).requires("CC",6)]);
        this.events.push([1500, Factory.B(0.8,1.5).requires("BB",2)]);
        this.events.push([1750, Factory.A(-1.2,-1).requires("AA",2)]);
      break;
      case 13:
        this.camera.smooth(2000).center(-0.6,0.4,3.6);
        this.events.push([1000, Factory.A(-2,-0.9).requires("AA",4)]);
        this.events.push([1250, Factory.AAA(-1.6,0).requires("AA",4).requires("B",16)]);
        this.events.push([1500, Factory.A(-1.2,0.8).requires("B",8)]);
        this.events.push([1750, Factory.B(-0.8,1.8).requires("BB",4)]);
        this.events.push([2000, Factory.BBB(-1.4,1.5).requires("AAA",1).requires("BB",4)]);
      break;
      case 14:
        this.camera.smooth(2000).center(-1,1.2,5);
        this.events.push([1000, Factory.A(1.1,-0.9).requires("CC",4)]);
        this.events.push([1100, Factory.A(1.1,0.8).requires("CC",4)]);
        this.events.push([1200, Factory.B(1.1,2.2).requires("BB",6)]);
        this.events.push([1300, Factory.B(1,3).requires("BB",4)]);
        this.events.push([1400, Factory.B(0.5,2.3).requires("CC",8)]);
        this.events.push([1500, Factory.A(0.3,3.1).requires("BB",4)]);
        this.events.push([1600, Factory.BB(-0.3,2.4).requires("CC",8)]);
        this.events.push([1700, Factory.A(-0.4,3.2).requires("BB",4)]);
        this.events.push([1800, Factory.B(-1,2.9).requires("BB",4)]);
        this.events.push([1900, Factory.AAA(-1.9,0.9).requires("BBB",1)]);
        this.events.push([2000, Factory.AA(-2.2,0.2).requires("AAA",2)]);
        this.events.push([2100, Factory.A(-2.6,-0.7).requires("CC",8)]);
        this.events.push([2200, Factory.A(-3.2,-0.9).requires("BBB",3)]);
        this.events.push([2300, Factory.A(-3,0).requires("BBB",3)]);
      break;
      case 15:
        this.camera.smooth(2000).center(-1.25,1.4,5.5);
        this.events.push([1000, Factory.CCC(-1.4,2.2).requires("AAA",2).requires("BBB",2)]);
        this.events.push([1100, Factory.AAA(-2,1.7).requires("AAA",4)]);
        this.events.push([1200, Factory.AA(-2.95,1.2).requires("AA",8)]);
        this.events.push([1300, Factory.A(-2.8,0.6).requires("A",16)]);
        this.events.push([1400, Factory.A(-3.6,1.2).requires("CCC",4)]);
        this.events.push([1500, Factory.A(-3.35,1.9).requires("CCC",4)]);
        this.events.push([1600, Factory.BB(-2.6,2.6).requires("CCC",2)]);
        this.events.push([1700, Factory.A(-2,2.35).requires("BB",8)]);
        this.events.push([1800, Factory.A(-2.55,1.9).requires("BB",8)]);
        this.events.push([1900, Factory.B(-3.15,2.6).requires("BB",8)]);
        this.events.push([2000, Factory.B(-3,3.3).requires("BB",8)]);
        this.events.push([2100, Factory.B(-2.35,3.3).requires("BB",8)]);
      break;
      case 16:
        this.camera.smooth(2000).center(-1.3,1.7,6);
        this.events.push([1000, Factory.B(1.2,3.8).requires("BB",8).requires("CCC",1)]);
        this.events.push([1250, Factory.B(0.6,4.1).requires("BBBB",18)]);
        this.events.push([1500, Factory.B(0,4.35).requires("BBBB",8)]);
        this.events.push([1750, Factory.A(-0.7,4.25).requires("BBBB",18)]);
        this.events.push([2000, Factory.A(-1.35,4.3).requires("BB",8).requires("CCC",1)]);
        this.events.push([2250, Factory.BB(-1.1,3.6).requires("BBBB",6)]);
        this.events.push([2500, Factory.BBBB(-1.75,3).requires("CCC",4)]);
      break;
      case 17:
        this.camera.smooth(2000).center(-1.7,1.9,6.5);
        this.events.push([1000, Factory.A(-4.1,2.1).requires("BBBB",24)]);
        this.events.push([1100, Factory.A(-4.4,2.8).requires("BBBB",24)]);
        this.events.push([1200, Factory.AA(-3.85,2.85).requires("BBBB",24)]);
        this.events.push([1300, Factory.A(-4,3.6).requires("BBBB",24)]);
        this.events.push([1400, Factory.A(-4.6,3.5).requires("AAAA",1)]);
        this.events.push([1500, Factory.A(-4.6,4.15).requires("AAAA",1)]);
        this.events.push([1600, Factory.A(-4.4,4.8).requires("AAAA",1)]);
        this.events.push([1700, Factory.AA(-3.8,4.4).requires("CCC",2)]);
        this.events.push([1800, Factory.AAA(-3.2,4.7).requires("CCC",2)]);
        this.events.push([1900, Factory.AAA(-3.2,4).requires("BBBB",24)]);
        this.events.push([2000, Factory.AAAA(-2.6,4.2).requires("CCC",4).requires("BBBB",24)]);
      break;
      case 18:
        this.events.push([1000, Factory.CCCC(-2,4).requires("AAAA",4).requires("BBBB",48).requires("CCC",4)]);
        this.events.push([1500, new Challenge(-2,4.8,"D").requires("CCCC",8)]);
      break;
      case 19:
        this.camera.smooth(2000).center(-2,4.8,7);
        this.events.push([SUCTION_FADE_IN_TIME, function () {
          thisWorld.camera.center(0,0,2);
          thisWorld.factories = [];
          thisWorld.background = new EndCard();
        }]);
      break;
    }
  }
  
  var i, d;
  for (i = 0, d = 0; i + d < this.events.length; ++i) {
    this.events[i][0] = this.events[i + d][0] - elapsed;
    if (d > 0) this.events[i][1] = this.events[i + d][1];
    if (this.events[i][0] <= 0) {
      if (this.events[i][1] instanceof Factory || this.events[i][1] instanceof Challenge) {
        SOUND.play(Game.audioContext, SOUND.appear.get());
        this.factories.push(this.events[i][1]); ++d; --i; 
      } else {
        this.events[i][1](); ++d; --i; 
      }
    }
  }
  this.events.length -= d;
  this.frames[this.frames.nextIndex] = elapsed;
  if (++this.frames.nextIndex > 10) this.frames.nextIndex = 0;;
};
World.prototype.draw = function (ctx) {
  // Clear the background
  ctx.fillStyle = 'black';
  ctx.fillRect(0,0,Game.WIDTH, Game.HEIGHT);
  if (this.background) this.background.draw(ctx);
  
  if (this.dragStart) {
    ctx.strokeStyle = "white";
    ctx.lineWidth = 2;
    ctx.beginPath();
    this.camera.pointToViewport(this.dragStart.x,this.dragStart.y,this.dragPoint);
    ctx.moveTo(this.dragPoint.x,this.dragPoint.y);
    ctx.lineTo(this.mouse.x, this.mouse.y);
    ctx.stroke();
  }
  
  var thisWorld = this;
  for (var i = 0; i < this.factories.length; ++i) {
    this.factories[i].preDraw(ctx, this.camera);
    this.factories[i].drawConnection(ctx);
  }
  for (var i = 0; i < this.factories.length; ++i) {
    this.factories[i].draw(ctx);
  }  
  for (var i = 0; i < this.effects.length; ++i) {
    this.effects[i].draw(ctx, this.camera);
  }  
  
  ctx.fillStyle = "white";
  if (this.hoverMute) {
    ctx.globalAlpha = 0.5;
    ctx.beginPath();
    ctx.arc(Game.WIDTH - buttonSize/2, Game.HEIGHT - buttonSize/2, buttonSize/2, 0, 2*Math.PI);
    ctx.fill();
    ctx.globalAlpha = 1;
  } else {
    ctx.globalAlpha = 0.25;
  }
  
  var bx = Game.WIDTH - buttonSize;
  var by = Game.HEIGHT - buttonSize;
  ctx.beginPath();
  ctx.moveTo(bx + 10, by + 25);
  ctx.lineTo(bx + 20, by + 25);
  ctx.lineTo(bx + 40, by + 10);
  ctx.lineTo(bx + 40, by + 70);
  ctx.lineTo(bx + 20, by + 55);
  ctx.lineTo(bx + 10, by + 55);
  ctx.fill();
  if (SOUND.isMuted) {
    ctx.beginPath();
    ctx.moveTo(bx + 45, by + 30);
    ctx.lineTo(bx + 50, by + 25);
    ctx.lineTo(bx + 60, by + 35);
    ctx.lineTo(bx + 70, by + 25);
    ctx.lineTo(bx + 75, by + 30);
    ctx.lineTo(bx + 65, by + 40);
    ctx.lineTo(bx + 75, by + 50);
    ctx.lineTo(bx + 70, by + 55);
    ctx.lineTo(bx + 60, by + 45);
    ctx.lineTo(bx + 50, by + 55);
    ctx.lineTo(bx + 45, by + 50);
    ctx.lineTo(bx + 55, by + 40);
    ctx.fill();
  } else {
    var cone = Math.PI/4;
    ctx.beginPath();
    ctx.arc(bx+40,by+40,10,cone,-cone,true);
    ctx.arc(bx+40,by+40,15,-cone,cone,false);
    ctx.arc(bx+40,by+40,20,cone,-cone,true);
    ctx.arc(bx+40,by+40,25,-cone,cone,false);
    ctx.arc(bx+40,by+40,30,cone,-cone,true);
    ctx.arc(bx+40,by+40,35,-cone,cone,false);
    ctx.fill();
  }
  ctx.globalAlpha = 1;
  
  if (this.hoverMuteMusic) {
    ctx.globalAlpha = 0.5;
    ctx.beginPath();
    ctx.arc(Game.WIDTH - 3*buttonSize/2, Game.HEIGHT - buttonSize/2, buttonSize/2, 0, 2*Math.PI);
    ctx.fill();
    ctx.globalAlpha = 1;
  } else {
    ctx.globalAlpha = 0.25;
  }
  
  bx = bx - buttonSize;
  drawMusicButton(ctx,bx,by);
  
  
  ctx.globalAlpha = 1;
  
  // ctx.textAlign = "left";
  // ctx.textBaseline = "top";
  // ctx.fillStyle = "white";
  // ctx.font = "16px sans-serif";
  // ctx.fillText("("+this.mouse.x+","+this.mouse.y+")", 4, 4);
  // ctx.fillText("("+Game.WIDTH+","+Game.HEIGHT+")", 4, 44);
  
  // if (this.dragStart) {
    // ctx.fillText("("+this.dragStart.x+","+this.dragStart.y+")", 4, 24);
  // }
  // if (this.frames.length >= 10) {
    // var avg = this.frames.reduce(function (acc, next) { return acc+next;})/this.frames.length;
    // var fps = Math.floor(1000/avg);
    // ctx.fillText(fps+" fps", 4, 64);
  // }
};
World.prototype.keydown = function (ev) {
  // var thisWorld = this;
  // this.effects.length = 0;
  // this.events.forEach(function (e) { 
    // if (e[1] instanceof Factory || e[1] instanceof Challenge) {
      // thisWorld.factories.push(e[1]); 
    // } else {
      // e[1]();
    // }
  // });
  // this.events.length = 0;
  // this.factories.forEach(function (f) { if (f.components) f.build(); });
  // SOUND.stopAll();
};
World.prototype.mousedown = function () {
  if (this.hoverMute) {
    if (SOUND.isMuted) {
      SOUND.unmute();
    } else {
      SOUND.mute();
    }
  } else if (this.hoverMuteMusic) {
    if (MUSIC.isMuted) {
      MUSIC.unmute();
    } else {
      MUSIC.mute();
    }
  }
  
  var dragStart = this.factories.find(function (f) { return f.hover });
  if (this.dragStart) {
    if (dragStart && dragStart !== this.dragStart) {
      // Connect
      SOUND.play(Game.audioContext, SOUND.endDrag.get());
      SOUND.stop(SOUND.holding.get());
      this.dragStart.connect(dragStart);
    } else {
      SOUND.play(Game.audioContext, SOUND.endDrag.get());
      SOUND.stop(SOUND.holding.get());
      this.dragStart.disconnect();
    }
    this.dragStart = null;
  } else if (dragStart && dragStart.connect) {
    SOUND.play(Game.audioContext, SOUND.startDrag.get());
    SOUND.play(Game.audioContext, SOUND.holding.get(), {loop:true});
    this.dragStart = dragStart;
  } else {
    this.dragStart = null;
  }
  this.dragPoint = {};
};
World.prototype.mouseup = function () {
  if (this.dragStart) {
    var dragEnd = this.factories.find(function (f) { return f.hover });
    if (dragEnd) {
      if (dragEnd !== this.dragStart) {
        // Connect
        SOUND.play(Game.audioContext, SOUND.endDrag.get());
        SOUND.stop(SOUND.holding.get());
        this.dragStart.connect(dragEnd);
        this.dragStart = null;
      }
    } else {
      SOUND.play(Game.audioContext, SOUND.endDrag.get());
      SOUND.stop(SOUND.holding.get());
      this.dragStart.disconnect();
      this.dragStart = null;
    }
  }
};
World.prototype.mousemove = function (ev) {
  this.mouse.x = ev.pageX - canvasX;
  this.mouse.y = ev.pageY - canvasY;
};

World.prototype.effect = function (effect) {
  var thisWorld = this;
  this.effects.push(effect);
  effect.onDie = function() { thisWorld.effects.splice(thisWorld.effects.indexOf(effect),1); };
};

var musicButtonBuffer;
var musicMuteBuffer;
function drawMusicButton(ctx, x, y) {
  var buttonSize = 80;
  var scale = buttonSize*0.75;
  if (!musicButtonBuffer) {
    musicButtonBuffer = document.createElement('canvas');
    musicButtonBuffer.width = buttonSize;
    musicButtonBuffer.height = buttonSize;
    var bufferCtx = musicButtonBuffer.getContext('2d');
    var bx = buttonSize/2;
    var by = buttonSize/2;
    bufferCtx.fillStyle = "white";
    bufferCtx.beginPath();
    bufferCtx.ellipse(bx-2.5*scale/10,by+3*scale/10,2*scale/10,3*scale/40,0,-1, 0, true);
    bufferCtx.lineTo(bx-0.25*scale/10,by-1.5*scale/10);
    bufferCtx.lineTo(bx+3.75*scale/10,by-2*scale/10);
    bufferCtx.ellipse(bx+2.5*scale/10,by+2*scale/10,2*scale/10,3*scale/40,0,-1, 0, true);
    bufferCtx.lineTo(bx+4.75*scale/10,by-4.25*scale/10);
    bufferCtx.lineTo(bx-1.15*scale/10,by-3.5*scale/10);
    bufferCtx.moveTo(bx-0.25*scale/10,by-2*scale/10);
    bufferCtx.lineTo(bx-0.20*scale/10,by-2.6*scale/10);
    bufferCtx.lineTo(bx+3.80*scale/10,by-3.1*scale/10);
    bufferCtx.lineTo(bx+3.75*scale/10,by-2.5*scale/10);
    bufferCtx.fill();
  }
  if (!musicMuteBuffer) {
    musicMuteBuffer = document.createElement('canvas');
    musicMuteBuffer.width = buttonSize;
    musicMuteBuffer.height = buttonSize;
    var bufferCtx = musicMuteBuffer.getContext('2d');
    bufferCtx.drawImage(musicButtonBuffer, 0, 0);
    bufferCtx.fillStyle = "rgba(0,0,0,1)";
    bufferCtx.globalCompositeOperation = "destination-out";
    bufferCtx.beginPath();
    bufferCtx.moveTo(64, 11);
    bufferCtx.lineTo(71, 14);
    bufferCtx.lineTo(14, 71);
    bufferCtx.lineTo(11, 64);
    bufferCtx.fill();
    bufferCtx.globalCompositeOperation = "source-over";
    bufferCtx.fillStyle = "white";
    bufferCtx.beginPath();
    bufferCtx.moveTo(66, 11);
    bufferCtx.lineTo(69, 14);
    bufferCtx.lineTo(14, 69);
    bufferCtx.lineTo(11, 66);
    bufferCtx.fill();
  }
  
  
  if (MUSIC.isMuted) {
    ctx.drawImage(musicMuteBuffer, x, y);
    // ctx.beginPath();
    // ctx.ellipse(bx-2.5*scale/10,by+3*scale/10,2*scale/10,3*scale/40,0,-1, 0, true);
    // ctx.lineTo(bx-0.25*scale/10,by-1.5*scale/10);
    // ctx.lineTo(bx+3.75*scale/10,by-2*scale/10);
    // ctx.ellipse(bx+2.5*scale/10,by+2*scale/10,2*scale/10,3*scale/40,0,-1, 0, true);
    // ctx.lineTo(bx+4.75*scale/10,by-4.25*scale/10);
    // ctx.lineTo(bx-1.15*scale/10,by-3.5*scale/10);
    // ctx.moveTo(bx-0.25*scale/10,by-2*scale/10);
    // ctx.lineTo(bx-0.20*scale/10,by-2.6*scale/10);
    // ctx.lineTo(bx+3.80*scale/10,by-3.1*scale/10);
    // ctx.lineTo(bx+3.75*scale/10,by-2.5*scale/10);
    // ctx.fill();
    // ctx.fillStyle = "red";
    // ctx.beginPath();
    // ctx.lineTo(bx + 25, by - 30);
    // ctx.lineTo(bx + 30, by - 25);
    // ctx.lineTo(bx - 25, by + 30);
    // ctx.lineTo(bx - 30, by + 25);
    // ctx.fill();
  } else {
    ctx.drawImage(musicButtonBuffer, x, y);
  }
};
